@@ -40,6 +40,7 @@
 typedef struct scan_results_t 
 {
         char           ssid[WLANCOND_MAX_SSID_SIZE+1];
+        gint32         ssid_len;
         unsigned char  bssid[ETH_ALEN];
         gint32         rssi;
         guint32        channel;
@@ -133,10 +134,12 @@ enum wlan_power_state {
         WLANCOND_POWER_ON = 1,
         WLANCOND_LONG_CAM,
         WLANCOND_SHORT_CAM,
+        WLANCOND_VERY_SHORT_CAM,
         WLANCOND_FULL_POWERSAVE
 };
 
-#define WLANCOND_DEFAULT_SLEEP_TIMEOUT 1000 // 1000ms
+#define WLANCOND_DEFAULT_SLEEP_TIMEOUT 200 // 200ms
+#define WLANCOND_VERY_SHORT_CAM_TIMEOUT 100 // 100ms
 #define WLANCOND_LONG_CAM_TIMEOUT 4000 // 4000ms
 
 #define MIC_FAILURE_TIMEOUT 60000 // 60 seconds
@@ -149,6 +152,7 @@ enum wlan_power_state {
 
 /* Gconf paths */
 #define SLEEP_GCONF_PATH "/system/osso/connectivity/IAP/wlan_sleep_timeout"
+#define INACTIVE_SLEEP_GCONF_PATH "/system/osso/connectivity/IAP/inactive_wlan_sleep_timeout"
 
 /* Common functions */
 gboolean monitor_wi(void);
