@@ -67,28 +67,12 @@ gchar own_mac[ETH_ALEN];
 static struct wlan_status_t wlan_status;
 static gboolean _flight_mode = FALSE;
 static gboolean _cover_closed = FALSE;
+static dbus_bool_t saved_inactivity = FALSE;
 
 /* Timer IDs */
 static guint wlan_if_down_timer_id = 0;
 static guint wlan_connect_timer_id = 0;
 
-//#define WPA_TEST_MODE
-#ifdef WPA_TEST_MODE
-const guint8 TEST_IE[] = 
-{
-        0xdd, 
-        0x16, 
-        0x00, 0x50, 0xf2, 0x01,
-        0x01, 0x00, 
-        0x00, 0x50, 0xf2, 0x02, 
-        0x01, 0x00, 
-        0x00, 0x50, 0xf2, 0x02, 
-        0x01, 0x00, 
-        0x00, 0x50, 0xf2, 0x02,
-};
-
-#endif
-
 /* These are for checking correct message parameters */
 
 static const char settings_and_connect_sgn[] = { 
@@ -320,15 +304,13 @@ static DBusHandlerResult mode_change_dbus(DBusMessage *message) {
 #ifdef ACTIVITY_CHECK
 static DBusHandlerResult activity_check_dbus(DBusMessage *message) {
         
-        dbus_bool_t activity;
-        
         if (!dbus_message_get_args(message, NULL,
-                                   DBUS_TYPE_BOOLEAN, &activity,
+                                   DBUS_TYPE_BOOLEAN, &saved_inactivity,
                                    DBUS_TYPE_INVALID)) {
                 DLOG_ERR("Invalid arguments for device_activity signal");
                 return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
         }
-        activity_check(activity);
+        activity_check(saved_inactivity);
         
         return DBUS_HANDLER_RESULT_HANDLED;
 }
@@ -346,21 +328,27 @@ void activity_check(dbus_bool_t inactivity) {
         if (get_wlan_state() != WLAN_CONNECTED) {
                 return;
         }
-
-        DLOG_DEBUG("WLAN activity mode changed to \"%s\"", 
-                   inactivity == TRUE?"inactive":"active");        
         
         if ((sock = socket_open()) <0)
         {
                 return;
         }
         if (inactivity == FALSE) {
+                //DLOG_DEBUG("WLAN activity mode changed to active");
                 set_power_state(WLANCOND_SHORT_CAM, sock);
         } else {
-                set_power_state(WLANCOND_FULL_POWERSAVE, sock);
+                //DLOG_DEBUG("WLAN activity mode changed to inactive");
+                set_power_state(WLANCOND_VERY_SHORT_CAM, sock);
         }
         close(sock);
 }
+/**
+ * Helper function for determining inactivity
+ */
+static gboolean get_inactivity_status(void) 
+{
+        return saved_inactivity;
+}
 #endif
 
 static DBusHandlerResult icd_check_signal_dbus(DBusMessage *message) {
@@ -369,6 +357,7 @@ static DBusHandlerResult icd_check_signal_dbus(DBusMessage *message) {
         char *icd_type;
         char *icd_state;
         char *icd_disconnect_reason;
+        guint ps_mode = WLANCOND_SHORT_CAM;
         DBusError dbus_error;
         int sock;
 
@@ -399,8 +388,11 @@ static DBusHandlerResult icd_check_signal_dbus(DBusMessage *message) {
                 if ((sock = socket_open()) >= 0) {
                         
                         DLOG_DEBUG("Going to power save");
-                        if (set_power_state(WLANCOND_SHORT_CAM,
-                                            sock) == FALSE) {
+
+                        if (get_inactivity_status() == TRUE)
+                                ps_mode = WLANCOND_VERY_SHORT_CAM;
+                        
+                        if (set_power_state(ps_mode, sock) == FALSE) {
                                 DLOG_ERR("Failed to set power save");
                         }
                         close(sock);
@@ -495,7 +487,6 @@ void set_wlan_state(int new_state, int send_signal, force_t force)
                 
                 if (get_wlan_state() != WLAN_NOT_INITIALIZED)
                         clear_wpa_mode();
-                
                 clean_ssid();
 
                 wlan_status.mode = 0;
@@ -615,10 +606,11 @@ int get_mode(void)
 }
 
 /** 
-    Helper function for resolving CAM timeout.
+    Helper function for resolving CAM timeout. 
+    @param path Gconf path to search for user specified value.
     @return CAM timeout.
 */
-static gint get_sleep_timeout(void) 
+static gint get_sleep_timeout(const gchar* path) 
 {
         GConfClient *client;
         GConfValue *gconf_value;
@@ -626,10 +618,10 @@ static gint get_sleep_timeout(void)
  
         client = gconf_client_get_default();
         if (client == NULL) {
-                return WLANCOND_DEFAULT_SLEEP_TIMEOUT;
+                return -1;
         }
 
-        gconf_value = gconf_client_get(client, SLEEP_GCONF_PATH, &error); 
+        gconf_value = gconf_client_get(client, path, &error); 
 
         if (error != NULL) {
                 DLOG_ERR("Could not get setting:%s, error:%s", 
@@ -638,12 +630,12 @@ static gint get_sleep_timeout(void)
                 
                 g_clear_error(&error);
                 g_object_unref(client);
-                return WLANCOND_DEFAULT_SLEEP_TIMEOUT;
+                return -1;
         }
         g_object_unref(client);
         
         if (gconf_value == NULL) {
-                return WLANCOND_DEFAULT_SLEEP_TIMEOUT;
+                return -1;
         }
         if (gconf_value->type == GCONF_VALUE_INT) {
                 gint sleep_timeout = gconf_value_get_int(gconf_value);
@@ -654,7 +646,7 @@ static gint get_sleep_timeout(void)
         }
         
         gconf_value_free(gconf_value);
-        return WLANCOND_DEFAULT_SLEEP_TIMEOUT;
+        return -1;
 }
 
 /**
@@ -666,26 +658,39 @@ static gint get_sleep_timeout(void)
 gboolean set_power_state(guint new_state, int sock) 
 {
         struct iwreq req;
+        gint sleep_timeout;
         
         if (wlan_status.power == new_state) {
                 return TRUE;
         }
         
         init_iwreq(&req);
-        
-        if (new_state == WLANCOND_POWER_ON) {
-                req.u.power.disabled = 1;        
-        } else if (new_state == WLANCOND_LONG_CAM) {
-		req.u.power.disabled = 0;
-		req.u.power.flags = IW_POWER_MULTICAST_R;
-                req.u.power.value = WLANCOND_LONG_CAM_TIMEOUT;
-        } else if (new_state == WLANCOND_SHORT_CAM) {
-		req.u.power.disabled = 0;
-		req.u.power.flags = IW_POWER_MULTICAST_R;
-                req.u.power.value = get_sleep_timeout();
-        } else {
-                req.u.power.disabled = 0;
-                req.u.power.flags = IW_POWER_ALL_R;
+
+        switch (new_state) {
+            case WLANCOND_POWER_ON:
+                    req.u.power.disabled = 1;
+                    break;
+            case WLANCOND_LONG_CAM:
+                    req.u.power.flags = IW_POWER_MULTICAST_R;
+                    req.u.power.value = WLANCOND_LONG_CAM_TIMEOUT;
+                    break;
+            case WLANCOND_SHORT_CAM:
+                    req.u.power.flags = IW_POWER_MULTICAST_R;
+                    sleep_timeout = get_sleep_timeout(SLEEP_GCONF_PATH);
+                    if (sleep_timeout < 0)
+                            sleep_timeout = WLANCOND_DEFAULT_SLEEP_TIMEOUT;
+                    req.u.power.value = sleep_timeout;
+                    break;
+            case WLANCOND_VERY_SHORT_CAM:
+                    req.u.power.flags = IW_POWER_MULTICAST_R;
+                    sleep_timeout = get_sleep_timeout(INACTIVE_SLEEP_GCONF_PATH);
+                    if (sleep_timeout < 0)
+                            sleep_timeout = WLANCOND_VERY_SHORT_CAM_TIMEOUT;
+                    req.u.power.value = sleep_timeout;
+                    break;
+            default:
+                    req.u.power.flags = IW_POWER_ALL_R;
+                    break;
         }
         
         if (ioctl(sock, SIOCSIWPOWER, &req) < 0) {
@@ -955,6 +960,7 @@ static DBusHandlerResult settings_and_connect_request(
         int key_len[4];
         int sock, i;
         int ssid_len;
+        int previous_state;
         DBusError derror;
 
 	dbus_error_init(&derror);
@@ -981,11 +987,13 @@ static DBusHandlerResult settings_and_connect_request(
                 return DBUS_HANDLER_RESULT_HANDLED;
         }
 
-        if (init_if(sock) < 0) {
+        if ((previous_state = init_if(sock)) < 0) {
                 reply = new_dbus_error(message, WLANCOND_ERROR_INIT_FAILED);
                 goto param_err;
         }
-
+        if (previous_state == WLAN_INITIALIZED_FOR_SCAN)
+                set_wlan_state(WLAN_INITIALIZED, NO_SIGNAL, FORCE_NO);
+        
         set_power_state(WLANCOND_POWER_ON, sock);
 
 	if (dbus_message_get_args(
@@ -1449,7 +1457,7 @@ int ask_scan_results(int ifindex)
                 
                 goto param_err;
         }
-        
+            
         if (req.u.data.length)
         {
                 struct iw_event            iwe;
@@ -1468,7 +1476,9 @@ int ask_scan_results(int ifindex)
                 do
                 {
                         /* Extract an event */
-                        ret = iw_extract_event_stream(&stream, &iwe, wireless_if->range.we_version_compiled);
+                        ret = iw_extract_event_stream(
+                                &stream, &iwe, 
+                                wireless_if->range.we_version_compiled);
                         if (ret > 0) {
                                 /* Let's peek what is coming so that we can
                                    separate different access points from
@@ -1569,6 +1579,7 @@ static DBusHandlerResult status_request(DBusMessage    *message,
         DBusMessage *reply = NULL;
         struct iwreq req;
         char *essid = NULL;
+        int essid_len;
         dbus_uint32_t sens = 0;
         dbus_uint32_t security = 0;
         dbus_uint32_t capability = 0;
@@ -1577,6 +1588,13 @@ static DBusHandlerResult status_request(DBusMessage    *message,
         unsigned char *key = NULL;
         int sock;
 
+        if (get_wlan_state() != WLAN_CONNECTED && 
+            get_wlan_state() != WLAN_NO_ADDRESS) {
+                reply = new_dbus_error(message, WLANCOND_ERROR_IOCTL_FAILED);
+                send_and_unref(connection, reply);
+                return DBUS_HANDLER_RESULT_HANDLED;
+        }
+                
         if ((sock = socket_open()) < 0)
         {
                 reply = new_dbus_error(message, WLANCOND_ERROR_FATAL);
@@ -1597,9 +1615,10 @@ static DBusHandlerResult status_request(DBusMessage    *message,
                 reply = new_dbus_error(message, WLANCOND_ERROR_IOCTL_FAILED);
                 goto param_err;
         }
-        if (!req.u.essid.flags) {
-                memset(essid, 0, sizeof(essid));
-        }
+        essid_len = req.u.essid.length;
+
+        if (essid_len == 0)
+                essid_len = 1;
         
         init_iwreq(&req);
 
@@ -1705,7 +1724,7 @@ static DBusHandlerResult status_request(DBusMessage    *message,
 
         append_dbus_args(reply,
                          DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, 
-                         &essid, strlen(essid)+1,
+                         &essid, essid_len,
                          DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, &bssid, ETH_ALEN,
                          DBUS_TYPE_UINT32, &sens,
                          DBUS_TYPE_UINT32, &channel,
@@ -1896,6 +1915,7 @@ static DBusHandlerResult set_powersave_request(DBusMessage    *message,
         int sock;
         DBusError error;
         dbus_bool_t onoff;
+        guint ps_mode = WLANCOND_SHORT_CAM;
         
         if ((sock = socket_open()) < 0)
         {
@@ -1924,7 +1944,9 @@ static DBusHandlerResult set_powersave_request(DBusMessage    *message,
                         set_wlan_state(WLAN_NOT_INITIALIZED, NO_SIGNAL, FORCE_YES);
                 } else if (get_wlan_state() != WLAN_NO_ADDRESS) {
                         // Go to power save if not acquiring address
-                        if (set_power_state(WLANCOND_SHORT_CAM, sock) == 
+                        if (get_inactivity_status() == TRUE)
+                                ps_mode = WLANCOND_VERY_SHORT_CAM;
+                        if (set_power_state(ps_mode, sock) == 
                             FALSE) {
                                 DLOG_ERR("Setting powersave failed");
                                 // Not fatal
@@ -1994,13 +2016,12 @@ int wpa_ie_push(unsigned char* ap_mac_addr, unsigned char* ap_wpa_ie,
         DBusMessage *msg;
         DBusPendingCall *pending;
 
-#ifndef WPA_TEST_MODE
         if (wlan_status.wpa_ie.ie_valid == IE_NOT_VALID || 
             selected_ssid == NULL) {
                 DLOG_ERR("WPA IE / SSID (%s) not valid", selected_ssid);
                 return -1;
         }
-#endif // WPA_TEST_MODE
+
         msg = dbus_message_new_method_call(
                 EAP_SERVICE,
                 EAP_REQ_PATH,
@@ -2011,7 +2032,6 @@ int wpa_ie_push(unsigned char* ap_mac_addr, unsigned char* ap_wpa_ie,
                 return -1;
         }
         
-#ifndef WPA_TEST_MODE
         append_dbus_args(
                 msg,
                 DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, 
@@ -2025,19 +2045,6 @@ int wpa_ie_push(unsigned char* ap_mac_addr, unsigned char* ap_wpa_ie,
                 DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, 
                 &ap_mac_addr, ETH_ALEN,
                 DBUS_TYPE_INVALID);
-#else
-        DLOG_DEBUG("Sending test IE");
-        append_dbus_args(msg,
-                         DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, 
-                         TEST_IE, sizeof(TEST_IE),
-                         DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, 
-                         TEST_IE, sizeof(TEST_IE),
-                         DBUS_TYPE_INT32, CIPHER_SUITE_TKIP,
-                         DBUS_TYPE_INT32, CIPHER_SUITE_TKIP,
-                         DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, 
-                         ap_mac_addr, ETH_ALEN,
-                         DBUS_TYPE_INVALID);
-#endif
 
         if (!dbus_connection_send_with_reply(get_dbus_connection(), 
                                              msg, &pending, -1))
