@@ -28,8 +28,6 @@
 #include <stdlib.h>
 #include <stdint.h>
 #include <string.h>
-#include <getopt.h>
-#include <signal.h>
 #include <glib.h>
 #include <glib-object.h>
 #include <errno.h>
@@ -60,9 +58,6 @@ struct rtnl_handle
 {
 	int			fd;
 	struct sockaddr_nl	local;
-	struct sockaddr_nl	peer;
-	__u32			seq;
-	__u32			dump;
 };
 
 /* Local functions */
@@ -72,6 +67,11 @@ static int handle_wpa_ie_event_binary(unsigned char* p, unsigned int length,
                                       struct scan_results_t *scan_results);
 static int handle_wpa_ie_assoc_event_binary(unsigned char* custom, unsigned int length);
 
+static inline void print_mac(const char *message, char* mac) 
+{
+        DLOG_DEBUG("%s %02x:%02x:%02x:%02x:%02x:%02x", message, mac[0],
+                   mac[1], mac[2], mac[3], mac[4], mac[5]);
+}
 
 /** 
     Remove saved scan results.
@@ -99,7 +99,8 @@ GSList *save_scan_results(struct scan_results_t *scan_results,
         
         //DLOG_DEBUG("\nScan results to save\n");
 
-        if (strnlen(scan_results->ssid, WLANCOND_MAX_SSID_SIZE) < 1) {
+        // ssid_len includes null termination
+        if (scan_results->ssid_len < 2) {
                 DLOG_DEBUG("Hidden SSID not saved to scan results");
                 // We have to free the result here since it is not saved
                 g_free(scan_results);
@@ -122,7 +123,7 @@ void send_dbus_scan_results(GSList *scan_results_save, const char* sender)
         GSList *list;
         int list_count = 0;
         
-        if (strnlen(sender, 5) == 0)
+        if (sender == NULL || strnlen(sender, 5) == 0)
                 return;
 
         DLOG_DEBUG("Sending scan results to DBUS to %s", sender);
@@ -158,8 +159,7 @@ void send_dbus_scan_results(GSList *scan_results_save, const char* sender)
                 if (!dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY, "y", &sub))
                         die("Out of memory during dbus_message_iter_open_container");
                 dbus_message_iter_append_fixed_array(
-                        &sub, DBUS_TYPE_BYTE, &p, 
-                        strlen(scan_results->ssid) + 1);
+                        &sub, DBUS_TYPE_BYTE, &p, scan_results->ssid_len);
                 if (!dbus_message_iter_close_container(&iter, &sub))
                         die("Out of memory during dbus_message_iter_close_container");
                 p = scan_results->bssid;
@@ -210,8 +210,7 @@ void disconnected_signal(void)
    @param scan_results Scan results to be sent.
    @param auth_status Authentication status.
 */
-static void connected_signal(struct scan_results_t *scan_results, 
-                             dbus_int32_t auth_status)
+static void connected_signal(unsigned char* bssid, dbus_int32_t auth_status)
 {
         DBusMessage *connected;
         
@@ -221,8 +220,6 @@ static void connected_signal(struct scan_results_t *scan_results,
                 WLANCOND_CONNECTED_SIG,
                 NULL);
         
-        unsigned char *bssid = scan_results->bssid;
-        
         append_dbus_args(connected,
                          DBUS_TYPE_STRING, &ifname,
                          DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, &bssid, ETH_ALEN, 
@@ -243,15 +240,36 @@ static void connected_signal(struct scan_results_t *scan_results,
 static void handle_wap_event(struct scan_results_t *scan_results, 
                              struct iw_event *event) 
 {
-        int sock, state;
+        int sock, state, scan_state;
+        gboolean zero_address = FALSE;
 
-        if (get_scan_state() != SCAN_NOT_ACTIVE || 
-            get_wlan_state() == WLAN_NOT_INITIALIZED) {
+        // Spurious event
+        if (get_wlan_state() == WLAN_NOT_INITIALIZED) {
+                return;
+        }
+
+        print_mac("SIOCGIWAP:", event->u.ap_addr.sa_data);
+        
+        if (!memcmp(event->u.ap_addr.sa_data, "\0\0\0\0\0\0", ETH_ALEN)) {
+                zero_address = TRUE;
+        }
+        
+        scan_state = get_scan_state();
+        
+        if (scan_state == SCAN_ACTIVE && zero_address && !get_mic_status()) {
+                DLOG_DEBUG("Got disconnected in the middle of scan");
+                set_wlan_state(WLAN_NOT_INITIALIZED,
+                               DISCONNECTED_SIGNAL,
+                               FORCE_YES);
+        }
+        
+        if (scan_state != SCAN_NOT_ACTIVE) {
+                memcpy(scan_results->bssid, event->u.ap_addr.sa_data, ETH_ALEN);
                 return;
         }
         
-        // Check if the address is not valid
-        if (memcmp(event->u.ap_addr.sa_data, "\0\0\0\0\0\0", ETH_ALEN))
+        // Check if the address is valid
+        if (zero_address == FALSE)
         {                
                 remove_connect_timer();
                 
@@ -272,13 +290,11 @@ static void handle_wap_event(struct scan_results_t *scan_results,
                         }
                 }
                 
-                connected_signal(scan_results, auth_status);
+                connected_signal(event->u.ap_addr.sa_data, auth_status);
                 // If not roaming, set NO_ADDRESS state
                 if (state != WLAN_CONNECTED) {
                         set_wlan_state(WLAN_NO_ADDRESS, NO_SIGNAL, FORCE_NO);
                 }
-
-                //set_wlan_state(WLAN_CONNECTED, NO_SIGNAL, FORCE_NO);
         } else {
                 
                 /* Set_wlan_state puts IF down */
@@ -288,14 +304,14 @@ static void handle_wap_event(struct scan_results_t *scan_results,
         }
 }
 
-
 /**
    Print or save wireless events.
    @param event extracted token.
    @param scan_results structure where scan results are saved.
+   @param ifindex Interface index.
    @return status.
  */
-int print_event_token(struct iw_event *event, 
+int print_event_token(struct iw_event *event,
                       struct scan_results_t *scan_results,
                       int ifindex)
 {
@@ -311,13 +327,14 @@ int print_event_token(struct iw_event *event,
                             len = WLANCOND_MAX_SSID_SIZE;
                     }
                     
-                    if((event->u.essid.pointer) && len) {
+                    if ((event->u.essid.pointer) && len) {
                             memcpy(scan_results->ssid, event->u.essid.pointer, 
                                    len);
                     }
-                    scan_results->ssid[len] = '\0';
+                    // Len includes null termination
+                    scan_results->ssid_len = len;
                     
-                    if(event->u.essid.flags)
+                    if (event->u.essid.flags)
                     {
                             /* Does it have an ESSID index ? */
                             if((event->u.essid.flags & IW_ENCODE_INDEX) > 1) {
@@ -340,8 +357,6 @@ int print_event_token(struct iw_event *event,
             }
             break;
             case SIOCGIWAP:
-                    memcpy(scan_results->bssid, event->u.ap_addr.sa_data, 6);
-                    DLOG_DEBUG("SIOCGIWAP: New Access Point: %02x:%02x:%02x:%02x:%02x:%02x", scan_results->bssid[0], scan_results->bssid[1], scan_results->bssid[2],scan_results->bssid[3],scan_results->bssid[4], scan_results->bssid[5]);                    
                     handle_wap_event(scan_results, event);
                     break;
             case IWEVQUAL:
@@ -478,12 +493,6 @@ int print_event_token(struct iw_event *event,
         return 0;
 }
 
-static inline void print_mac(const char *message, char* mac) 
-{
-        DLOG_DEBUG("%s %02x:%02x:%02x:%02x:%02x:%02x", message, mac[0],
-                   mac[1], mac[2], mac[3], mac[4], mac[5]);
-}
-
 /**
    Handle association time WPA IE event.
    @param p WPA IE data.
@@ -736,9 +745,9 @@ static int print_event_stream(int ifindex, char *data, int len)
 {
         struct iw_event	iwe;
         struct stream_descr stream;
-        int ret;
         struct wireless_iface *wireless_if;
         struct scan_results_t scan_results;
+        int ret;
         
         wireless_if = get_interface_data(ifindex);
 
